@@ -1,5 +1,6 @@
-#include <core.h>
-#include "decls.h"
+#include <cheapr/internal/c_core.h>
+#include "internal_helpers.h"
+#include "declarations.h"
 #include <R_ext/Rdynload.h> // For DllInfo on R 3.3
 
 // -----------------------------------------------------------------------------
@@ -19,7 +20,7 @@ api_set_add_attrs(SEXP x, SEXP attributes, bool add){
   try {
     return cpp_set_add_attributes(x, attributes, add);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -28,7 +29,7 @@ api_set_rm_attrs(SEXP x) {
   try {
     return cpp_set_rm_attributes(x);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -37,7 +38,7 @@ api_exclude_locs(SEXP exclude, R_xlen_t xn) {
   try {
     return exclude_locs(exclude, xn);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -51,11 +52,11 @@ api_unnested_length(SEXP x){
 }
 
 SEXP
-api_drop_null(SEXP l, bool always_shallow_copy) {
+api_drop_null(SEXP l) {
   try {
-    return cpp_drop_null(l, always_shallow_copy);
+    return cpp_drop_null(l);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -64,15 +65,7 @@ api_lengths(SEXP x, bool names){
   try {
     return cpp_lengths(x, names);
   } catch (...) {
-    return R_NilValue;
-  }
-}
-SEXP
-api_new_list(R_xlen_t length, SEXP default_value){
-  try {
-    return new_list(length, default_value);
-  } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -81,7 +74,7 @@ api_list_assign(SEXP x, SEXP values){
   try {
     return cpp_list_assign(x, values);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -90,7 +83,7 @@ api_sset(SEXP x, SEXP indices, bool check){
   try {
     return cpp_sset(x, indices, check);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -99,7 +92,7 @@ api_sset_vec(SEXP x, SEXP indices, bool check){
   try {
     return sset_vec(x, indices, check);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -108,16 +101,16 @@ api_val_find(SEXP x, SEXP value, bool invert){
   try {
     return cpp_which_val(x, value, invert);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
 SEXP
 api_val_remove(SEXP x, SEXP value){
   try {
-    return cpp_val_remove(x, value);
+    return cpp_val_remove(x, value, false);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -126,7 +119,7 @@ api_loc_set_replace(SEXP x, SEXP where, SEXP what){
   try {
     return cpp_loc_set_replace(x, where, what);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -135,7 +128,7 @@ api_sequence(SEXP size, SEXP from, SEXP by, bool as_list, bool add_id){
   try {
     return cpp_sequence(size, from, by, as_list, add_id);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -144,7 +137,7 @@ api_seq_len(R_xlen_t n){
   try {
     return cpp_seq_len(n);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -153,7 +146,16 @@ api_rep_len(SEXP x, int length){
   try {
     return cpp_rep_len(x, length);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
+  }
+}
+
+SEXP
+api_rep_len2(SEXP x, R_xlen_t length){
+  try {
+    return cpp_rep_len(x, length);
+  } catch (...) {
+    return cheapr::r_null;
   }
 }
 
@@ -162,7 +164,7 @@ api_rep(SEXP x, SEXP times){
   try {
     return cpp_rep(x, times);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -171,7 +173,7 @@ api_rep_each(SEXP x, SEXP each){
   try {
     return cpp_rep_each(x, each);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -180,7 +182,7 @@ api_recycle(SEXP x, SEXP length){
   try {
     return cpp_recycle(x, length);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -189,7 +191,7 @@ api_c(SEXP x){
   try {
     return cpp_c(x);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -198,7 +200,7 @@ api_list_c(SEXP x){
   try {
     return cpp_list_c(x);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -207,7 +209,7 @@ api_name_repair(SEXP names, SEXP dup_sep, SEXP empty_sep){
   try {
     return cpp_name_repair(names, dup_sep, empty_sep);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -216,7 +218,7 @@ api_unique(SEXP x, bool names){
   try {
     return cpp_unique(x, names);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -225,7 +227,7 @@ api_setdiff(SEXP x, SEXP y, bool unique){
   try {
     return cpp_setdiff(x, y, unique);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -234,7 +236,7 @@ api_intersect(SEXP x, SEXP y, bool unique){
   try {
     return cpp_intersect(x, y, unique);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -243,27 +245,18 @@ api_get_ptype(SEXP x){
   try {
     return get_ptype(x);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
 // Data frame functions
 
-SEXP
-api_create_df_row_names(int n) {
-  try {
-    return create_df_row_names(n);
-  } catch (...) {
-    return R_NilValue;
-  }
-}
-
 SEXP
 api_list_as_df(SEXP x){
   try {
     return cpp_list_as_df(x);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -272,7 +265,7 @@ api_new_df(SEXP x, SEXP nrows, bool recycle, bool name_repair){
   try {
     return cpp_new_df(x, nrows, recycle, name_repair);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -281,7 +274,7 @@ api_df_slice(SEXP x, SEXP indices, bool check){
   try {
     return cpp_df_slice(x, indices, check);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -290,7 +283,7 @@ api_df_select(SEXP x, SEXP locs){
   try {
     return cpp_df_select(x, locs);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -299,7 +292,7 @@ api_df_subset(SEXP x, SEXP i, SEXP j, bool check){
   try {
     return cpp_df_subset(x, i, j, check);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -308,7 +301,7 @@ api_df_assign_cols(SEXP x, SEXP cols){
   try {
     return cpp_df_assign_cols(x, cols);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -317,7 +310,7 @@ api_df_col_c(SEXP x, bool recycle, bool name_repair){
   try {
     return cpp_df_col_c(x, recycle, name_repair);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -326,7 +319,7 @@ api_str_coalesce(SEXP x){
   try {
     return cpp_str_coalesce(x);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -335,7 +328,7 @@ api_rebuild(SEXP x, SEXP source, bool shallow_copy){
   try {
     return rebuild(x, source, shallow_copy);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -344,23 +337,23 @@ api_semi_copy(SEXP x){
   try {
     return cpp_semi_copy(x);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
 SEXP
-api_paste(SEXP x, SEXP sep, SEXP collapse){
+api_paste(SEXP list_of_chars, SEXP sep, SEXP collapse){
   try {
-    return cpp_paste(x, sep, collapse);
+    return cpp_paste(list_of_chars, sep, collapse);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
 void
 api_replace(SEXP x, SEXP where, SEXP with, bool quiet){
   try {
-    static_cast<void>(cpp_replace(x, where, with, true, quiet));
+    replace_in_place(x, where, with, quiet);
   } catch (...) {
   }
 }
@@ -370,7 +363,7 @@ api_if_else(SEXP condition, SEXP yes, SEXP no, SEXP na){
   try {
     return cpp_if_else(condition, yes, no, na);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
@@ -379,7 +372,33 @@ api_gcd(SEXP x, double tol, bool na_rm){
   try {
     return cpp_gcd(x, tol, na_rm, true, true);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
+  }
+}
+
+SEXP
+api_clean_indices(SEXP locs, SEXP x){
+  try {
+    return clean_locs(locs, x);
+  } catch (...) {
+    return cheapr::r_null;
+  }
+}
+
+SEXP
+api_get_names(SEXP x){
+  try {
+    return get_vec_names(x);
+  } catch (...) {
+    return cheapr::r_null;
+  }
+}
+
+void
+api_set_names(SEXP x, SEXP names){
+  try {
+    set_vec_names(x, names);
+  } catch (...) {
   }
 }
 
@@ -388,7 +407,7 @@ api_gcd(SEXP x, double tol, bool na_rm){
 R_xlen_t
 api_vec_length(SEXP x) {
   try {
-    return cheapr::vector_length(x);
+    return cheapr::vec::length(x);
   } catch (...) {
     return (R_xlen_t) 0;
   }
@@ -399,22 +418,14 @@ api_r_address(SEXP x) {
   try {
     return cheapr::address(x);
   } catch (...) {
-    return R_NilValue;
+    return cheapr::r_null;
   }
 }
 
 bool
-api_is_simple_atomic_vec(SEXP x){
+api_is_simple_atomic_vec(SEXP x) {
   try {
-    return cheapr::cheapr_is_simple_atomic_vec(x);
-  } catch (...) {
-    return false;
-  }
-}
-bool
-api_is_simple_vec(SEXP x){
-  try {
-    return cheapr::cheapr_is_simple_vec(x);
+    return cheapr::internal::cheapr_is_simple_atomic_vec(x);
   } catch (...) {
     return false;
   }
@@ -425,7 +436,6 @@ api_is_simple_vec(SEXP x){
 [[cpp11::init]]
 void api_init(DllInfo* dll){
   R_RegisterCCallable("cheapr", "api_is_compact_seq",    (DL_FUNC)api_is_compact_seq);
-  R_RegisterCCallable("cheapr", "api_create_df_row_names",    (DL_FUNC)api_create_df_row_names);
   R_RegisterCCallable("cheapr", "api_set_add_attrs",    (DL_FUNC)api_set_add_attrs);
   R_RegisterCCallable("cheapr", "api_set_rm_attrs",    (DL_FUNC)api_set_rm_attrs);
   R_RegisterCCallable("cheapr", "api_exclude_locs",    (DL_FUNC)api_exclude_locs);
@@ -434,7 +444,6 @@ void api_init(DllInfo* dll){
   R_RegisterCCallable("cheapr", "api_list_as_df",    (DL_FUNC)api_list_as_df);
   R_RegisterCCallable("cheapr", "api_new_df",    (DL_FUNC)api_new_df);
   R_RegisterCCallable("cheapr", "api_lengths",    (DL_FUNC)api_lengths);
-  R_RegisterCCallable("cheapr", "api_new_list",    (DL_FUNC)api_new_list);
   R_RegisterCCallable("cheapr", "api_list_assign",    (DL_FUNC)api_list_assign);
   R_RegisterCCallable("cheapr", "api_df_slice",    (DL_FUNC)api_df_slice);
   R_RegisterCCallable("cheapr", "api_df_select",    (DL_FUNC)api_df_select);
@@ -448,6 +457,7 @@ void api_init(DllInfo* dll){
   R_RegisterCCallable("cheapr", "api_seq_len",    (DL_FUNC)api_seq_len);
   R_RegisterCCallable("cheapr", "api_recycle",    (DL_FUNC)api_recycle);
   R_RegisterCCallable("cheapr", "api_rep_len",    (DL_FUNC)api_rep_len);
+  R_RegisterCCallable("cheapr", "api_rep_len2",    (DL_FUNC)api_rep_len2);
   R_RegisterCCallable("cheapr", "api_rep",    (DL_FUNC)api_rep);
   R_RegisterCCallable("cheapr", "api_rep_each",    (DL_FUNC)api_rep_each);
   R_RegisterCCallable("cheapr", "api_c",    (DL_FUNC)api_c);
@@ -468,6 +478,8 @@ void api_init(DllInfo* dll){
   R_RegisterCCallable("cheapr", "api_gcd",    (DL_FUNC)api_gcd);
   R_RegisterCCallable("cheapr", "api_vec_length",    (DL_FUNC)api_vec_length);
   R_RegisterCCallable("cheapr", "api_r_address",    (DL_FUNC)api_r_address);
+  R_RegisterCCallable("cheapr", "api_clean_indices",    (DL_FUNC)api_clean_indices);
   R_RegisterCCallable("cheapr", "api_is_simple_atomic_vec",    (DL_FUNC)api_is_simple_atomic_vec);
-  R_RegisterCCallable("cheapr", "api_is_simple_vec",    (DL_FUNC)api_is_simple_vec);
+  R_RegisterCCallable("cheapr", "api_get_names",    (DL_FUNC)api_get_names);
+  R_RegisterCCallable("cheapr", "api_set_names",    (DL_FUNC)api_set_names);
 }
