@@ -18,7 +18,6 @@ SEXP lag(SEXP x, R_xlen_t k, SEXP fill, bool set) {
   }
 
   bool set_and_altrep = set && ALTREP(x);
-  SEXP out;
   if (ALTREP(x)){
     set = true;
   }
@@ -26,251 +25,86 @@ SEXP lag(SEXP x, R_xlen_t k, SEXP fill, bool set) {
   if (set_and_altrep){
     Rf_warning("Cannot lag an ALTREP by reference, a copy has been made.\n\tEnsure the result is assigned to an object if used in further calculations\n\te.g. `x <- lag_(x, set = TRUE)`");
   }
-  switch(CHEAPR_TYPEOF(xvec)){
-  case NILSXP: {
-    out = R_NilValue;
-    break;
-  }
-  case LGLSXP:
-  case INTSXP: {
-    k = k >= 0 ? std::min(size, k) : std::max(-size, k);
-    out = SHIELD(set ? xvec : cpp_semi_copy(xvec)); ++NP;
-    int* RESTRICT p_out = INTEGER(out);
-    int *p_x = INTEGER(xvec);
-    SHIELD(fill = cast<r_integer_t>(fill, R_NilValue)); ++NP;
-    auto fill_value = fill_size > 0 ? INTEGER(fill)[0] : na_type(p_x[0]);
-    if (k >= 0){
-      safe_memmove(&p_out[k], &p_x[0], (size - k) * sizeof(int));
-      OMP_FOR_SIMD
-      for (R_xlen_t i = 0; i < k; ++i) p_out[i] = fill_value;
-    } else {
-      safe_memmove(&p_out[0], &p_x[-k], (size + k) * sizeof(int));
-      OMP_FOR_SIMD
-      for (R_xlen_t i = size - 1; i >= size + k; --i) p_out[i] = fill_value;
-    }
-    break;
-  }
-  case CHEAPR_INT64SXP: {
-    k = k >= 0 ? std::min(size, k) : std::max(-size, k);
-    out = SHIELD(set ? xvec : cpp_semi_copy(xvec)); ++NP;
-    int64_t* RESTRICT p_out = INTEGER64_PTR(out);
-    int64_t *p_x = INTEGER64_PTR(xvec);
-    SHIELD(fill = cast<r_integer64_t>(fill, R_NilValue)); ++NP;
-    auto fill_value = fill_size > 0 ? INTEGER64_PTR(fill)[0] : na_type(p_x[0]);
+  k = k >= 0 ? std::min(size, k) : std::max(-size, k);
 
-    if (k >= 0){
-      safe_memmove(&p_out[k], &p_x[0], (size - k) * sizeof(int64_t));
-      OMP_FOR_SIMD
-      for (R_xlen_t i = 0; i < k; ++i) p_out[i] = fill_value;
-    } else {
-      safe_memmove(&p_out[0], &p_x[-k], (size + k) * sizeof(int64_t));
-      OMP_FOR_SIMD
-      for (R_xlen_t i = size - 1; i >= size + k; --i) p_out[i] = fill_value;
-    }
-    break;
-  }
-  case REALSXP: {
-    k = k >= 0 ? std::min(size, k) : std::max(-size, k);
-    out = SHIELD(set ? xvec : cpp_semi_copy(xvec)); ++NP;
-    double* RESTRICT p_out = REAL(out);
-    double *p_x = REAL(xvec);
-    SHIELD(fill = cast<r_numeric_t>(fill, R_NilValue)); ++NP;
-    auto fill_value = fill_size > 0 ? REAL(fill)[0] : na_type(p_x[0]);
+  SEXP out = visit_vector(xvec, [&](auto p_x) -> SEXP {
 
-    if (k >= 0){
-      safe_memmove(&p_out[k], &p_x[0], (size - k) * sizeof(double));
-      OMP_FOR_SIMD
-      for (R_xlen_t i = 0; i < k; ++i) p_out[i] = fill_value;
-    } else {
-      safe_memmove(&p_out[0], &p_x[-k], (size + k) * sizeof(double));
-      OMP_FOR_SIMD
-      for (R_xlen_t i = size - 1; i >= size + k; --i) p_out[i] = fill_value;
-    }
-    break;
-  }
-  case CPLXSXP: {
-    k = k >= 0 ? std::min(size, k) : std::max(-size, k);
-    out = SHIELD(set ? xvec : cpp_semi_copy(xvec)); ++NP;
-    Rcomplex *p_out = COMPLEX(out);
-    Rcomplex *p_x = COMPLEX(xvec);
-    SHIELD(fill = cast<r_complex_t>(fill, R_NilValue)); ++NP;
-    auto fill_value = fill_size > 0 ? COMPLEX(fill)[0] : na_type(p_x[0]);
+    // Data type of vector elements
+    using data_t = std::remove_pointer_t<decltype(p_x)>;
 
-    if (k >= 0){
-      safe_memmove(&p_out[k], &p_x[0], (size - k) * sizeof(Rcomplex));
-      for (R_xlen_t i = 0; i < k; ++i) SET_COMPLEX_ELT(out, i, fill_value);
-    } else {
-      safe_memmove(&p_out[0], &p_x[-k], (size + k) * sizeof(Rcomplex));
-      OMP_FOR_SIMD
-      for (R_xlen_t i = size - 1; i >= size + k; --i) SET_COMPLEX_ELT(out, i, fill_value);
-    }
-    break;
-  }
-  case STRSXP: {
-    k = k >= 0 ? std::min(size, k) : std::max(-size, k);
-    out = SHIELD(set ? xvec : cpp_semi_copy(xvec)); ++NP;
-    const SEXP *p_out = STRING_PTR_RO(out);
-    const SEXP *p_x = STRING_PTR_RO(xvec);
-
-    SHIELD(fill = cast<r_character_t>(fill, R_NilValue)); ++NP;
-    auto fill_value = fill_size > 0 ? STRING_ELT(fill, 0) : na_type(p_x[0]);
-
-    if (set){
-      R_xlen_t tempi;
-      // If k = 0 then no lag occurs
-      if (std::abs(k) >= 1){
-        SEXP lag_temp = SHIELD(new_vec(STRSXP, std::abs(k)));++NP;
-        SEXP tempv = SHIELD(new_vec(STRSXP, 1)); ++NP;
-        const SEXP *p_lag = STRING_PTR_RO(lag_temp);
-        // Positive lags
-        if (k >= 0){
-          for (R_xlen_t i = 0; i < k; ++i) {
-            SET_STRING_ELT(lag_temp, i, p_out[i]);
-            SET_STRING_ELT(out, i, fill_value);
-          }
-          for (R_xlen_t i = k; i < size; ++i) {
-            tempi = ((i - k) % k);
-            SET_STRING_ELT(tempv, 0, p_lag[tempi]);
-            SET_STRING_ELT(lag_temp, tempi, p_out[i]);
-            SET_STRING_ELT(out, i, STRING_ELT(tempv, 0));
-          }
-          // Negative lags
-        } else {
-          for (R_xlen_t i = size - 1; i >= size + k; --i) {
-            SET_STRING_ELT(lag_temp, size - i - 1, p_out[i]);
-            SET_STRING_ELT(out, i, fill_value);
-          }
-          for (R_xlen_t i = size + k - 1; i >= 0; --i) {
-            tempi = ( (size - (i - k) - 1) % k);
-            SET_STRING_ELT(tempv, 0, p_lag[tempi]);
-            SET_STRING_ELT(lag_temp, tempi, p_out[i]);
-            SET_STRING_ELT(out, i, STRING_ELT(tempv, 0));
-          }
-        }
+    if constexpr (std::is_same_v<data_t, std::nullptr_t>) {
+      if (is_null(xvec)){
+        return r_null;
       }
-    } else {
+      Rf_error("%s cannot handle an object of type %s", __func__, Rf_type2char(TYPEOF(x)));
+    } else if constexpr (is_r_ptr_writable_v<data_t>){
+      using writable_data_t = std::remove_const_t<data_t>;
+      SEXP res = SHIELD(set ? xvec : cpp_semi_copy(xvec)); ++NP;
+      auto *p_res = vector_ptr<writable_data_t>(res);
+      SHIELD(fill = cast_(get_r_type(xvec), fill, r_null)); ++NP;
+      auto fill_value = fill_size > 0 ? get_value<writable_data_t>(fill, 0) : na_value<data_t>();
+
       if (k >= 0){
-        for (R_xlen_t i = 0; i < k; ++i) SET_STRING_ELT(out, i, fill_value);
-        for (R_xlen_t i = k; i < size; ++i) SET_STRING_ELT(out, i, p_x[i - k]);
+        safe_memmove(&p_res[k], &p_x[0], sizeof(data_t) * (size - k));
+        r_fill(res, p_res, 0, k, fill_value);
       } else {
-        for (R_xlen_t i = size - 1; i >= size + k; --i) SET_STRING_ELT(out, i, fill_value);
-        for (R_xlen_t i = size + k - 1; i >= 0; --i) SET_STRING_ELT(out, i, p_x[i - k]);
-      }
-    }
-    break;
-  }
-  case RAWSXP: {
-    k = k >= 0 ? std::min(size, k) : std::max(-size, k);
-    out = SHIELD(set ? xvec : cpp_semi_copy(xvec)); ++NP;
-    Rbyte *p_out = RAW(out);
-    const Rbyte *p_x = RAW_RO(xvec);
-
-    SHIELD(fill = cast<r_raw_t>(fill, R_NilValue)); ++NP;
-    auto fill_value = fill_size > 0 ? RAW(fill)[0] : na_type(p_x[0]);
-
-    if (set){
-      R_xlen_t tempi;
-      // If k = 0 then no lag occurs
-      if (std::abs(k) >= 1){
-        SEXP lag_temp = SHIELD(new_vec(RAWSXP, std::abs(k)));
-        ++NP;
-        Rbyte *p_lag = RAW(lag_temp);
-        // Positive lags
-        if (k >= 0){
-          for (R_xlen_t i = 0; i < k; ++i) {
-            SET_RAW_ELT(lag_temp, i, p_out[i]);
-            SET_RAW_ELT(out, i, fill_value);
-          }
-          for (R_xlen_t i = k; i < size; ++i) {
-            tempi = ((i - k) % k);
-            Rbyte tempv = p_lag[tempi];
-            SET_RAW_ELT(lag_temp, tempi, p_out[i]);
-            SET_RAW_ELT(out, i, tempv);
-          }
-          // Negative lags
-        } else {
-          for (R_xlen_t i = size - 1; i >= size + k; --i) {
-            SET_RAW_ELT(lag_temp, size - i - 1, p_out[i]);
-            SET_RAW_ELT(out, i, fill_value);
-          }
-          for (R_xlen_t i = size + k - 1; i >= 0; --i) {
-            tempi = ( (size - (i - k) - 1) % k);
-            Rbyte tempv = p_lag[tempi];
-            SET_RAW_ELT(lag_temp, tempi, p_out[i]);
-            SET_RAW_ELT(out, i, tempv);
-          }
-        }
+        safe_memmove(&p_res[0], &p_x[-k], sizeof(data_t) * (size + k));
+        r_fill(res, p_res, size + k, -k, fill_value);
       }
+      return res;
     } else {
-      if (k >= 0){
-        for (R_xlen_t i = 0; i < size; ++i) {
-          SET_RAW_ELT(out, i, i >= k ? p_x[i - k] : fill_value);
+      SEXP res = SHIELD(set ? xvec : cpp_semi_copy(xvec)); ++NP;
+      auto *p_res = vector_ptr<data_t>(res);
+      SHIELD(fill = cast_(get_r_type(xvec), fill, r_null)); ++NP;
+      auto fill_value = fill_size > 0 ? get_value<data_t>(fill, 0) : na_value<data_t>();
+
+      if (set){
+        R_xlen_t tempi;
+        // If k = 0 then no lag occurs
+        if (std::abs(k) >= 1){
+          SEXP lag_temp = SHIELD(new_vector<std::decay_t<data_t>>(std::abs(k))); ++NP;
+          std::decay_t<data_t> temp_value;
+          auto *p_lag = vector_ptr<data_t>(lag_temp);
+          // Positive lags
+          if (k >= 0){
+            for (R_xlen_t i = 0; i < k; ++i) {
+              set_value(lag_temp, i, p_res[i]);
+              set_value(res, i, fill_value);
+            }
+            for (R_xlen_t i = k; i < size; ++i) {
+              tempi = ((i - k) % k);
+              temp_value = p_lag[tempi];
+              set_value(lag_temp, tempi, p_res[i]);
+              set_value(res, i, temp_value);
+            }
+            // Negative lags
+          } else {
+            for (R_xlen_t i = size - 1; i >= size + k; --i) {
+              set_value(lag_temp, size - i - 1, p_res[i]);
+              set_value(res, i, fill_value);
+            }
+            for (R_xlen_t i = size + k - 1; i >= 0; --i) {
+              tempi = ( (size - (i - k) - 1) % k);
+              temp_value = p_lag[tempi];
+              set_value(lag_temp, tempi, p_res[i]);
+              set_value(res, i, temp_value);
+            }
+          }
         }
       } else {
-        for (R_xlen_t i = size - 1; i >= 0; --i) {
-          SET_RAW_ELT(out, i, (i - size) < k ? p_x[i - k] : fill_value);
-        }
-      }
-    }
-    break;
-  }
-  case VECSXP: {
-    k = k >= 0 ? std::min(size, k) : std::max(-size, k);
-    SEXP fill_value = SHIELD(coerce_vec(fill_size >= 1 ? fill : R_NilValue, VECSXP)); ++NP;
-    out = SHIELD(set ? xvec : new_vec(VECSXP, size)); ++NP;
-    const SEXP *p_out = LIST_PTR_RO(out);
-    if (set){
-      R_xlen_t tempi;
-      // If k = 0 then no lag occurs
-      if (std::abs(k) >= 1){
-        SEXP lag_temp = SHIELD(new_vec(VECSXP, std::abs(k))); ++NP;
-        SEXP tempv = SHIELD(new_vec(VECSXP, 1)); ++NP;
-        const SEXP *p_lag = LIST_PTR_RO(lag_temp);
-        // Positive lags
         if (k >= 0){
-          for (R_xlen_t i = 0; i < k; ++i) {
-            SET_VECTOR_ELT(lag_temp, i, p_out[i]);
-            SET_VECTOR_ELT(out, i, VECTOR_ELT(fill_value, 0));
+          for (R_xlen_t i = 0; i < size; ++i) {
+            set_value(res, i, i >= k ? p_x[i - k] : fill_value);
           }
-          for (R_xlen_t i = k; i < size; ++i) {
-            tempi = ((i - k) % k);
-            SET_VECTOR_ELT(tempv, 0, p_lag[tempi]);
-            SET_VECTOR_ELT(lag_temp, tempi, p_out[i]);
-            SET_VECTOR_ELT(out, i, VECTOR_ELT(tempv, 0));
-          }
-          // Negative lags
         } else {
-          for (R_xlen_t i = size - 1; i >= size + k; --i) {
-            SET_VECTOR_ELT(lag_temp, size - i - 1, p_out[i]);
-            SET_VECTOR_ELT(out, i, VECTOR_ELT(fill_value, 0));
-          }
-          for (R_xlen_t i = size + k - 1; i >= 0; --i) {
-            tempi = ( (size - (i - k) - 1) % k);
-            SET_VECTOR_ELT(tempv, 0, p_lag[tempi]);
-            SET_VECTOR_ELT(lag_temp, tempi, p_out[i]);
-            SET_VECTOR_ELT(out, i, VECTOR_ELT(tempv, 0));
+          for (R_xlen_t i = size - 1; i >= 0; --i) {
+            set_value(res, i, (i - size) < k ? p_x[i - k] : fill_value);
           }
         }
       }
-    } else {
-      const SEXP *p_x = LIST_PTR_RO(xvec);
-      if (k >= 0){
-        for (R_xlen_t i = 0; i < size; ++i) {
-          SET_VECTOR_ELT(out, i, i >= k ? p_x[i - k] : VECTOR_ELT(fill_value, 0));
-        }
-      } else {
-        for (R_xlen_t i = size - 1; i >= 0; --i) {
-          SET_VECTOR_ELT(out, i, (i - size) < k ? p_x[i - k] : VECTOR_ELT(fill_value, 0));
-        }
-      }
+      return res;
     }
-    break;
-  }
-  default: {
-    YIELD(NP);
-    Rf_error("%s cannot handle an object of type %s", __func__, Rf_type2char(TYPEOF(xvec)));
-  }
-  }
+  });
   YIELD(NP);
   return out;
 }
@@ -280,22 +114,25 @@ SEXP lag(SEXP x, R_xlen_t k, SEXP fill, bool set) {
 
 [[cpp11::register]]
 SEXP cpp_lag(SEXP x, R_xlen_t k, SEXP fill, bool set, bool recursive){
-  int32_t NP = 0;
-  SEXP out = R_NilValue;
+  SEXP out = r_null;
   if (recursive && TYPEOF(x) == VECSXP){
     R_xlen_t size = Rf_xlength(x);
-    const SEXP *p_x = LIST_PTR_RO(x);
-    out = SHIELD(new_vec(VECSXP, size)); ++NP;
+    const SEXP *p_x = list_ptr_ro(x);
+    out = SHIELD(new_list(size));
     SHALLOW_DUPLICATE_ATTRIB(out, x);
     for (R_xlen_t i = 0; i < size; ++i){
-      SET_VECTOR_ELT(out, i, cpp_lag(p_x[i], k, fill, set && !ALTREP(p_x[i]), true));
+      SEXP elem = SHIELD(cpp_lag(p_x[i], k, fill, set && !ALTREP(p_x[i]), true));
+      SET_VECTOR_ELT(out, i, elem);
+      YIELD(1);
     }
+    YIELD(1);
   } else {
-    out = SHIELD(lag(x, k, fill, set)); ++NP;
-    SEXP names = SHIELD(get_names(x)); ++NP;
-    set_names(out, lag(names, k, fill, set && !ALTREP(x)));
+    out = SHIELD(lag(x, k, fill, set));
+    SEXP names = SHIELD(get_old_names(x));
+    SEXP lagged_names = SHIELD(lag(names, k, fill, set && !ALTREP(x)));
+    set_old_names(out, lagged_names);
+    YIELD(3);
   }
-  YIELD(NP);
   return out;
 }
 
@@ -323,15 +160,15 @@ SEXP lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill){
   // hence order/run_lengths should remain NULL throughout each recursion
   // if they are NULL
 
-  SHIELD(lag = coerce_vec(lag, INTSXP)); ++NP;
-  SHIELD(order = has_order ? coerce_vec(order, INTSXP) : R_NilValue); ++NP;
-  SHIELD(run_lengths = has_rl ? coerce_vec(run_lengths, INTSXP) : R_NilValue); ++NP;
+  SHIELD(lag = vec::coerce_vec(lag, INTSXP)); ++NP;
+  SHIELD(order = has_order ? vec::coerce_vec(order, INTSXP) : r_null); ++NP;
+  SHIELD(run_lengths = has_rl ? vec::coerce_vec(run_lengths, INTSXP) : r_null); ++NP;
   int foo = 42;
   const int *p_o = &foo;
   const int *p_rl = &foo;
-  if (has_order) p_o = INTEGER(order);
-  if (has_rl) p_rl = INTEGER(run_lengths);
-  const int *p_lag = INTEGER(lag);
+  if (has_order) p_o = integer_ptr(order);
+  if (has_rl) p_rl = integer_ptr(run_lengths);
+  const int *p_lag = integer_ptr(lag);
   int rl; // Run-length
   int run_start = 0; // Start index of current run
   int run_end = 0; // End index of current run
@@ -340,7 +177,7 @@ SEXP lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill){
   // Manually set run rl size to 1 if run_lengths = NULL (checked prior)
   if (!has_rl) rl_size = 1;
   bool recycle_lag = lag_size != 1;
-  SEXP out = R_NilValue;
+  SEXP out = r_null;
   switch(CHEAPR_TYPEOF(x)){
   case NILSXP: {
     break;
@@ -351,13 +188,14 @@ SEXP lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill){
     if (has_order && (size != o_size)){
       Rf_error("length(order) must equal length(x) (%d)", size);
     }
-    const int *p_x = INTEGER(x);
-    int fill_value = NA_INTEGER;
+    const int *p_x = integer_ptr(x);
+    int fill_value = na::integer;
     if (fill_size >= 1){
-      fill_value = Rf_asInteger(fill);
+      SEXP temp_fill = SHIELD(cast<r_integers_t>(fill, r_null)); ++NP;
+      fill_value = integer_ptr(temp_fill)[0];
     }
     out = SHIELD(cpp_semi_copy(x)); ++NP;
-    int* RESTRICT p_out = INTEGER(out);
+    int* RESTRICT p_out = integer_ptr(out);
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
       // Manually set run rl if order = NULL
@@ -405,14 +243,14 @@ SEXP lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill){
     if (has_order && (size != o_size)){
       Rf_error("length(order) must equal length(x) (%d)", size);
     }
-    const int64_t *p_x = INTEGER64_PTR_RO(x);
-    int64_t fill_value = NA_INTEGER64;
+    const int64_t *p_x = integer64_ptr_ro(x);
+    int64_t fill_value = na::integer64;
     if (fill_size >= 1){
-      SEXP temp_fill = SHIELD(cast<r_integer64_t>(fill, R_NilValue)); ++NP;
-      fill_value = INTEGER64_PTR(temp_fill)[0];
+      SEXP temp_fill = SHIELD(cast<r_integers64_t>(fill, r_null)); ++NP;
+      fill_value = integer64_ptr(temp_fill)[0];
     }
     out = SHIELD(cpp_semi_copy(x)); ++NP;
-    int64_t* RESTRICT p_out = INTEGER64_PTR(out);
+    int64_t* RESTRICT p_out = integer64_ptr(out);
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
       rl = has_rl ? p_rl[i] : size; // Current run-length
@@ -459,13 +297,14 @@ SEXP lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill){
     if (has_order && (size != o_size)){
       Rf_error("length(order) must equal length(x) (%d)", size);
     }
-    const double *p_x = REAL(x);
-    double fill_value = NA_REAL;
+    const double *p_x = real_ptr(x);
+    double fill_value = na::real;
     if (fill_size >= 1){
-      fill_value = Rf_asReal(fill);
+      SEXP temp_fill = SHIELD(cast<r_doubles_t>(fill, r_null)); ++NP;
+      fill_value = real_ptr(temp_fill)[0];
     }
     out = SHIELD(cpp_semi_copy(x)); ++NP;
-    double* RESTRICT p_out = REAL(out);
+    double* RESTRICT p_out = real_ptr(out);
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
       rl = has_rl ? p_rl[i] : size; // Current run-length
@@ -512,8 +351,9 @@ SEXP lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill){
     if (has_order && (size != o_size)){
       Rf_error("length(order) must equal length(x) (%d)", size);
     }
-    const SEXP *p_x = STRING_PTR_RO(x);
-    SEXP fill_value = SHIELD(fill_size >= 1 ? Rf_asChar(fill) : NA_STRING); ++NP;
+    const r_string_t *p_x = string_ptr_ro(x);
+    SEXP temp_fill = SHIELD(cast<r_characters_t>(fill, r_null)); ++NP;
+    r_string_t fill_value = fill_size >= 1 ? get_value<r_string_t>(temp_fill, 0) : na::string;
     out = SHIELD(cpp_semi_copy(x)); ++NP;
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
@@ -544,9 +384,9 @@ SEXP lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill){
         }
         k = p_lag[recycle_lag ? oi % lag_size : 0];
         if (k >= 0){
-          SET_STRING_ELT(out, oi, (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
+          set_value<r_string_t>(out, oi, (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
         } else {
-          SET_STRING_ELT(out, oi, (j - run_end) < k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
+          set_value<r_string_t>(out, oi, (j - run_end) < k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
         }
       }
     }
@@ -561,12 +401,9 @@ SEXP lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill){
     if (has_order && (size != o_size)){
       Rf_error("length(order) must equal length(x) (%d)", size);
     }
-    Rcomplex *p_x = COMPLEX(x);
-    SEXP fill_sexp = SHIELD(new_vec(CPLXSXP, 1)); ++NP;
-    Rcomplex *p_fill = COMPLEX(fill_sexp);
-    p_fill[0].i = NA_REAL;
-    p_fill[0].r = NA_REAL;
-    Rcomplex fill_value = fill_size >= 1 ? Rf_asComplex(fill) : COMPLEX(fill_sexp)[0];
+    r_complex_t *p_x = complex_ptr(x);
+    SEXP temp_fill = SHIELD(cast<r_complexes_t>(fill, r_null)); ++NP;
+    r_complex_t fill_value = fill_size >= 1 ? get_value<r_complex_t>(temp_fill, 0) : na::complex;
     out = SHIELD(cpp_semi_copy(x)); ++NP;
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
@@ -597,9 +434,9 @@ SEXP lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill){
         }
         k = p_lag[recycle_lag ? oi % lag_size : 0];
         if (k >= 0){
-          SET_COMPLEX_ELT(out, oi, (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
+          set_value<r_complex_t>(out, oi, (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
         } else {
-          SET_COMPLEX_ELT(out, oi, (j - run_end) < k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
+          set_value<r_complex_t>(out, oi, (j - run_end) < k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
         }
       }
     }
@@ -614,9 +451,9 @@ SEXP lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill){
     if (has_order && (size != o_size)){
       Rf_error("length(order) must equal length(x) (%d)", size);
     }
-    Rbyte *p_x = RAW(x);
-    SEXP raw_sexp = SHIELD(coerce_vec(fill, RAWSXP));
-    Rbyte fill_value = fill_size == 0 ? RAW(Rf_ScalarRaw(0))[0] : RAW(raw_sexp)[0]; ++NP;
+    r_byte_t *p_x = raw_ptr(x);
+    SEXP raw_sexp = SHIELD(vec::coerce_vec(fill, RAWSXP)); ++NP;
+    r_byte_t fill_value = fill_size == 0? r_byte_t{0} : raw_ptr(raw_sexp)[0];
     out = SHIELD(cpp_semi_copy(x)); ++NP;
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
@@ -647,9 +484,9 @@ SEXP lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill){
         }
         k = p_lag[recycle_lag ? oi % lag_size : 0];
         if (k >= 0){
-          SET_RAW_ELT(out, oi, (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
+          set_value<r_byte_t>(out, oi, (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
         } else {
-          SET_RAW_ELT(out, oi, (j - run_end) < k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
+          set_value<r_byte_t>(out, oi, (j - run_end) < k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
         }
       }
     }
@@ -664,9 +501,13 @@ SEXP lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill){
     if (has_order && (size != o_size)){
       Rf_error("length(order) must equal length(x) (%d)", size);
     }
-    const SEXP *p_x = LIST_PTR_RO(x);
-    SEXP fill_value = SHIELD(VECTOR_ELT(coerce_vec(fill_size >= 1 ? fill : R_NilValue, VECSXP), 0)); ++NP;
-    out = SHIELD(new_vec(VECSXP, size)); ++NP;
+    const SEXP *p_x = list_ptr_ro(x);
+    SEXP fill_value = r_null;
+    if (fill_size >= 1){
+      SEXP temp_fill = SHIELD(cast<r_list_t>(fill, r_null)); ++NP;
+      fill_value = get_value<SEXP>(temp_fill, 0);
+    }
+    out = SHIELD(new_list(size)); ++NP;
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
       rl = has_rl ? p_rl[i] : size; // Current run-length
@@ -720,21 +561,24 @@ SEXP lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill){
 
 [[cpp11::register]]
 SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool recursive){
-  int32_t NP = 0;
-  SEXP out = R_NilValue;
+  SEXP out = r_null;
   if (recursive && TYPEOF(x) == VECSXP){
     R_xlen_t size = Rf_xlength(x);
-    const SEXP *p_x = LIST_PTR_RO(x);
-    out = SHIELD(new_vec(VECSXP, size)); ++NP;
+    const SEXP *p_x = list_ptr_ro(x);
+    out = SHIELD(new_list(size));
     SHALLOW_DUPLICATE_ATTRIB(out, x);
     for (R_xlen_t i = 0; i < size; ++i){
-      SET_VECTOR_ELT(out, i, cpp_lag2(p_x[i], lag, order, run_lengths, fill, true));
+      SEXP elem = SHIELD(cpp_lag2(p_x[i], lag, order, run_lengths, fill, true));
+      SET_VECTOR_ELT(out, i, elem);
+      YIELD(1);
     }
+    YIELD(1);
   } else {
-    SEXP names = SHIELD(get_names(x)); ++NP;
-    out = SHIELD(lag2(x, lag, order, run_lengths, fill)); ++NP;
-    set_names(out, lag2(names, lag, order, run_lengths, fill));
+    SEXP names = SHIELD(get_old_names(x));
+    out = SHIELD(lag2(x, lag, order, run_lengths, fill));
+    SEXP lagged_names = SHIELD(lag2(names, lag, order, run_lengths, fill));
+    set_old_names(out, lagged_names);
+    YIELD(3);
   }
-  YIELD(NP);
   return out;
 }
