@@ -3,6 +3,7 @@
 // NA handling functions
 // Author: Nick Christofides
 
+
 #define CHEAPR_ANY_NA                                            \
 for (R_xlen_t i = 0; i < n; ++i){                                \
   if (is_r_na(p_x[i])){                                          \
@@ -19,97 +20,96 @@ for (R_xlen_t i = 0; i < n; ++i){                                \
   }                                                              \
 }
 
-#define CHEAPR_IS_NA                                               \
-if (n_cores > 1){                                                  \
-  OMP_PARALLEL_FOR_SIMD                                            \
-  for (R_xlen_t i = 0; i < n; ++i){                                \
-    p_out[i] = is_r_na(p_x[i]);                                    \
-  }                                                                \
-} else {                                                           \
-  OMP_FOR_SIMD                                                     \
-  for (R_xlen_t i = 0; i < n; ++i){                                \
-    p_out[i] = is_r_na(p_x[i]);                                    \
-  }                                                                \
+#define CHEAPR_IS_NA                                                          \
+if (n_threads > 1){                                                           \
+  OMP_PARALLEL_FOR_SIMD(n_threads)                                            \
+  for (R_xlen_t i = 0; i < n; ++i){                                           \
+    set_value(p_out, i, static_cast<r_bool_t>(is_r_na(p_x[i])));                \
+  }                                                                           \
+} else {                                                                      \
+  OMP_SIMD                                                                    \
+  for (R_xlen_t i = 0; i < n; ++i){                                           \
+    set_value(p_out, i, static_cast<r_bool_t>(is_r_na(p_x[i])));                \
+  }                                                                           \
 }
 
-#define CHEAPR_NA_COUNT                                                   \
-if (do_parallel){                                                         \
-  _Pragma("omp parallel for simd num_threads(n_cores) reduction(+:count)")\
-  for (R_xlen_t i = 0; i < n; ++i) count += is_r_na(p_x[i]);              \
-} else {                                                                  \
-  OMP_FOR_SIMD                                                            \
-  for (R_xlen_t i = 0; i < n; ++i) count += is_r_na(p_x[i]);              \
+
+bool vec_any(SEXP x){
+  R_xlen_t n = Rf_xlength(x);
+  const r_bool_t *p_x = logical_ptr_ro(x);
+
+  bool out = false;
+  for (R_xlen_t i = 0; i < n; ++i){
+    if (is_r_true(p_x[i])){
+     out = true;
+      break;
+    }
+  }
+  return out;
 }
 
+bool vec_all(SEXP x){
+  R_xlen_t n = Rf_xlength(x);
+  const r_bool_t *p_x = logical_ptr_ro(x);
+
+  bool out = true;
+
+  for (R_xlen_t i = 0; i < n; ++i){
+    if (p_x[i] == r_false){
+      out = false;
+      break;
+    }
+  }
+  return out;
+}
 
 R_xlen_t na_count(SEXP x, bool recursive){
   R_xlen_t n = Rf_xlength(x);
   R_xlen_t count = 0;
   int32_t NP = 0;
-  int n_cores = n >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
-  bool do_parallel = n_cores > 1;
-  switch ( CHEAPR_TYPEOF(x) ){
-  case NILSXP: {
-    break;
-  }
-  case LGLSXP:
-  case INTSXP: {
-    const int *p_x = INTEGER_RO(x);
-    CHEAPR_NA_COUNT
-    break;
-  }
-  case CHEAPR_INT64SXP: {
-    const int64_t *p_x = INTEGER64_PTR_RO(x);
-    CHEAPR_NA_COUNT
-    break;
-  }
-  case REALSXP: {
-    const double *p_x = REAL_RO(x);
-    CHEAPR_NA_COUNT
-    break;
-  }
-  case STRSXP: {
-    const SEXP *p_x = STRING_PTR_RO(x);
-    CHEAPR_NA_COUNT
-    break;
-  }
-  case RAWSXP: {
-    break;
-  }
-  case CPLXSXP: {
-    const Rcomplex *p_x = COMPLEX_RO(x);
-    CHEAPR_NA_COUNT
-    break;
-  }
-  case VECSXP: {
-    // We use a recursive method if recursive is true
-    // Otherwise we skip to the default section below
-    if (recursive){
-    const SEXP *p_x = LIST_PTR_RO(x);
-    for (R_xlen_t i = 0; i < n; ++i){
-      count += na_count(p_x[i], true);
+  int n_threads = calc_threads(n);
+  visit_vector(x, [&](auto p_x) {
+
+    using data_t = std::remove_const_t<std::remove_pointer_t<std::decay_t<decltype(p_x)>>>;
+
+    auto default_scalar_count = [&] {
+      SEXP is_missing = SHIELD(eval_pkg_fun("is_na", "cheapr", env::base_env, x)); ++NP;
+      SEXP scalar_true = SHIELD(as_vector(r_true)); ++NP;
+      count = scalar_count(is_missing, scalar_true, true);
+    };
+
+    if constexpr (std::is_same_v<data_t, std::nullptr_t>){
+      if (is_null(x)){
+        return;
+      } else {
+        default_scalar_count();
+      }
+    } else if constexpr (std::is_same_v<data_t, SEXP>) {
+      if (recursive){
+        R_CheckStack(); // Check C Stack size isn't close to the limit
+        for (R_xlen_t i = 0; i < n; ++i){
+          count += na_count(p_x[i], true);
+        }
+      } else {
+        default_scalar_count();
+      }
+    } else {
+      if (n_threads > 1){
+        _Pragma("omp parallel for simd num_threads(n_threads) reduction(+:count)")
+        for (R_xlen_t i = 0; i < n; ++i) count += is_r_na(p_x[i]);
+      } else {
+        _Pragma("omp simd reduction(+:count)")
+        for (R_xlen_t i = 0; i < n; ++i) count += is_r_na(p_x[i]);
+      }
     }
-    break;
-  } else {
-    const SEXP *p_x = LIST_PTR_RO(x);
-    CHEAPR_NA_COUNT
-    break;
-  }
-  }
-  default: {
-    SEXP is_missing = SHIELD(cheapr_is_na(x)); ++NP;
-    SEXP r_true = SHIELD(as_r_scalar(true)); ++NP;
-    count = scalar_count(is_missing, r_true, true);
-    break;
-  }
-  }
+  });
   YIELD(NP);
   return count;
 }
 
 [[cpp11::register]]
 SEXP cpp_num_na(SEXP x, bool recursive){
-  return as_r_scalar(na_count(x, recursive));
+  return as_vector(na_count(x, recursive));
 }
 
 
@@ -124,22 +124,22 @@ bool cpp_any_na(SEXP x, bool recursive){
   }
   case LGLSXP:
   case INTSXP: {
-    int *p_x = INTEGER(x);
+    int *p_x = integer_ptr(x);
     CHEAPR_ANY_NA;
     break;
   }
   case CHEAPR_INT64SXP: {
-    int64_t *p_x = INTEGER64_PTR(x);
+    int64_t *p_x = integer64_ptr(x);
     CHEAPR_ANY_NA;
     break;
   }
   case REALSXP: {
-    double *p_x = REAL(x);
+    double *p_x = real_ptr(x);
     CHEAPR_ANY_NA;
     break;
   }
   case STRSXP: {
-    const SEXP *p_x = STRING_PTR_RO(x);
+    const r_string_t *p_x = string_ptr_ro(x);
     CHEAPR_ANY_NA;
     break;
   }
@@ -147,12 +147,13 @@ bool cpp_any_na(SEXP x, bool recursive){
     break;
   }
   case CPLXSXP: {
-    Rcomplex *p_x = COMPLEX(x);
+    r_complex_t *p_x = complex_ptr(x);
     CHEAPR_ANY_NA;
     break;
   }
   case VECSXP: {
     if (recursive){
+    R_CheckStack(); // Check C Stack size isn't close to the limit
     for (int i = 0; i < n; ++i){
       out = cpp_any_na(VECTOR_ELT(x, i), true);
       if (out) break;
@@ -161,9 +162,8 @@ bool cpp_any_na(SEXP x, bool recursive){
   }
   }
   default: {
-    SEXP is_missing = SHIELD(cheapr_is_na(x)); ++NP;
-    SEXP any_missing = SHIELD(cpp11::package("base")["any"](is_missing)); ++NP;
-    out = Rf_asLogical(any_missing);
+    SEXP is_missing = SHIELD(eval_pkg_fun("is_na", "cheapr", env::base_env, x)); ++NP;
+    out = vec_any(is_missing);
     break;
   }
   }
@@ -186,22 +186,22 @@ bool cpp_all_na(SEXP x, bool return_true_on_empty, bool recursive){
   switch ( CHEAPR_TYPEOF(x) ){
   case LGLSXP:
   case INTSXP: {
-    int *p_x = INTEGER(x);
+    int *p_x = integer_ptr(x);
     CHEAPR_ALL_NA;
     break;
   }
   case CHEAPR_INT64SXP: {
-    int64_t *p_x = INTEGER64_PTR(x);
+    int64_t *p_x = integer64_ptr(x);
     CHEAPR_ALL_NA;
     break;
   }
   case REALSXP: {
-    double *p_x = REAL(x);
+    double *p_x = real_ptr(x);
     CHEAPR_ALL_NA;
     break;
   }
   case STRSXP: {
-    const SEXP *p_x = STRING_PTR_RO(x);
+    const r_string_t *p_x = string_ptr_ro(x);
     CHEAPR_ALL_NA;
     break;
   }
@@ -209,12 +209,13 @@ bool cpp_all_na(SEXP x, bool return_true_on_empty, bool recursive){
     break;
   }
   case CPLXSXP: {
-    Rcomplex *p_x = COMPLEX(x);
+    r_complex_t *p_x = complex_ptr(x);
     CHEAPR_ALL_NA;
     break;
   }
   case VECSXP: {
     if (recursive){
+    R_CheckStack(); // Check C Stack size isn't close to the limit
     for (int i = 0; i < n; ++i){
       out = cpp_all_na(VECTOR_ELT(x, i), return_true_on_empty, true);
       if (!out) break;
@@ -223,9 +224,8 @@ bool cpp_all_na(SEXP x, bool return_true_on_empty, bool recursive){
   }
   }
   default: {
-    SEXP is_missing = SHIELD(cheapr_is_na(x)); ++NP;
-    SEXP all_missing = SHIELD(cpp11::package("base")["all"](is_missing)); ++NP;
-    out = Rf_asLogical(all_missing);
+    SEXP is_missing = SHIELD(eval_pkg_fun("is_na", "cheapr", env::base_env, x)); ++NP;
+    out = vec_all(is_missing);
     break;
   }
   }
@@ -240,67 +240,61 @@ bool cpp_all_na(SEXP x, bool return_true_on_empty, bool recursive){
 [[cpp11::register]]
 SEXP cpp_is_na(SEXP x){
   R_xlen_t n = Rf_xlength(x);
-  int n_cores = n >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
+  int n_threads = calc_threads(n);
   SEXP out;
   switch ( CHEAPR_TYPEOF(x) ){
   case NILSXP: {
-    out = SHIELD(new_vec(LGLSXP, 0));
+    out = SHIELD(new_vector<r_bool_t>(0));
     break;
   }
   case LGLSXP:
   case INTSXP: {
-    out = SHIELD(new_vec(LGLSXP, n));
-    int* RESTRICT p_out = LOGICAL(out);
-    const int *p_x = INTEGER(x);
+    out = SHIELD(new_vector<r_bool_t>(n));
+    r_bool_t* RESTRICT p_out = logical_ptr(out);
+    const int *p_x = integer_ptr(x);
     CHEAPR_IS_NA
     break;
   }
   case CHEAPR_INT64SXP: {
-    out = SHIELD(new_vec(LGLSXP, n));
-    int* RESTRICT p_out = LOGICAL(out);
-    const int64_t *p_x = INTEGER64_PTR_RO(x);
+    out = SHIELD(new_vector<r_bool_t>(n));
+    r_bool_t* RESTRICT p_out = logical_ptr(out);
+    const int64_t *p_x = integer64_ptr_ro(x);
     CHEAPR_IS_NA
     break;
   }
   case REALSXP: {
-    out = SHIELD(new_vec(LGLSXP, n));
-    int* RESTRICT p_out = LOGICAL(out);
-    const double *p_x = REAL(x);
+    out = SHIELD(new_vector<r_bool_t>(n));
+    r_bool_t* RESTRICT p_out = logical_ptr(out);
+    const double *p_x = real_ptr(x);
     CHEAPR_IS_NA
     break;
   }
   case STRSXP: {
-    out = SHIELD(new_vec(LGLSXP, n));
-    int* RESTRICT p_out = LOGICAL(out);
-    const SEXP *p_x = STRING_PTR_RO(x);
+    out = SHIELD(new_vector<r_bool_t>(n));
+    r_bool_t* RESTRICT p_out = logical_ptr(out);
+    const r_string_t *p_x = string_ptr_ro(x);
     CHEAPR_IS_NA
     break;
   }
   case RAWSXP: {
-    out = SHIELD(new_vec(LGLSXP, n));
-    int* RESTRICT p_out = LOGICAL(out);
-    std::fill(p_out, p_out + n, 0);
+    out = SHIELD(new_vector<r_bool_t>(n, r_false));
     break;
   }
   case CPLXSXP: {
-    out = SHIELD(new_vec(LGLSXP, n));
-    int* RESTRICT p_out = LOGICAL(out);
-    const Rcomplex *p_x = COMPLEX(x);
+    out = SHIELD(new_vector<r_bool_t>(n));
+    r_bool_t* RESTRICT p_out = logical_ptr(out);
+    const r_complex_t *p_x = complex_ptr(x);
     CHEAPR_IS_NA
     break;
   }
   case VECSXP: {
-    if (!Rf_isObject(x)){
-    out = SHIELD(new_vec(LGLSXP, n));
-    int* RESTRICT p_out = LOGICAL(out);
-    const SEXP *p_x = LIST_PTR_RO(x);
-    CHEAPR_IS_NA
+    if (!vec::is_object(x)){
+    out = SHIELD(new_vector<r_bool_t>(n, r_false));
     break;
   }
   }
   default: {
-    out = SHIELD(cpp11::package("base")["is.na"](x));
-    break;
+    return eval_pkg_fun("is.na", "base", env::base_env, x);
   }
   }
   YIELD(1);
@@ -312,43 +306,42 @@ SEXP cpp_df_row_na_counts(SEXP x){
   if (!is_df(x)){
     Rf_error("x must be a data frame");
   }
-  const SEXP *p_x = LIST_PTR_RO(x);
+  const SEXP *p_x = list_ptr_ro(x);
   int32_t NP = 0;
   int num_col = Rf_length(x);
-  int num_row = df_nrow(x);
-  SEXP out = SHIELD(new_vec(INTSXP, num_row)); ++NP;
-  int* RESTRICT p_out = INTEGER(out);
-  std::fill(p_out, p_out + num_row, 0);
+  int num_row = df::nrow(x);
+  SEXP out = SHIELD(vec::new_vector<int>(num_row, 0)); ++NP;
+  int* RESTRICT p_out = integer_ptr(out);
   for (int j = 0; j < num_col; ++j){
     switch ( CHEAPR_TYPEOF(p_x[j]) ){
     case LGLSXP:
     case INTSXP: {
-      const int *p_xj = INTEGER_RO(p_x[j]);
-      OMP_FOR_SIMD
+      const int *p_xj = integer_ptr_ro(p_x[j]);
+      OMP_SIMD
       for (int i = 0; i < num_row; ++i){
         p_out[i] += is_r_na(p_xj[i]);
       }
       break;
     }
     case CHEAPR_INT64SXP: {
-      const int64_t *p_xj = INTEGER64_PTR_RO(p_x[j]);
-      OMP_FOR_SIMD
+      const int64_t *p_xj = integer64_ptr_ro(p_x[j]);
+      OMP_SIMD
       for (int i = 0; i < num_row; ++i){
         p_out[i] += is_r_na(p_xj[i]);
       }
      break;
     }
     case REALSXP: {
-      const double *p_xj = REAL_RO(p_x[j]);
-      OMP_FOR_SIMD
+      const double *p_xj = real_ptr_ro(p_x[j]);
+      OMP_SIMD
       for (int i = 0; i < num_row; ++i){
         p_out[i] += is_r_na(p_xj[i]);
       }
       break;
     }
     case STRSXP: {
-      const SEXP *p_xj = STRING_PTR_RO(p_x[j]);
-      OMP_FOR_SIMD
+      const r_string_t *p_xj = string_ptr_ro(p_x[j]);
+      OMP_SIMD
       for (int i = 0; i < num_row; ++i){
         p_out[i] += is_r_na(p_xj[i]);
       }
@@ -358,30 +351,31 @@ SEXP cpp_df_row_na_counts(SEXP x){
       break;
     }
     case CPLXSXP: {
-      const Rcomplex *p_xj = COMPLEX_RO(p_x[j]);
-      OMP_FOR_SIMD
+      const r_complex_t *p_xj = complex_ptr_ro(p_x[j]);
+      OMP_SIMD
       for (int i = 0; i < num_row; ++i){
         p_out[i] += is_r_na(p_xj[i]);
       }
       break;
     }
     case VECSXP: {
-      if (Rf_isObject(p_x[j])){
-      SEXP is_missing = SHIELD(cheapr_is_na(p_x[j])); ++NP;
+      if (vec::is_object(p_x[j])){
+
+      SEXP is_missing = SHIELD(eval_pkg_fun("is_na", "cheapr", env::base_env, p_x[j])); ++NP;
       if (Rf_length(is_missing) != num_row){
         int int_nrows = num_row;
         int element_length = Rf_length(is_missing); ++NP;
-        SEXP names = SHIELD(get_names(x));
+        SEXP names = SHIELD(get_old_names(x));
         YIELD(NP);
         Rf_error("is.na method for list variable %s produces a length (%d) vector which does not equal the number of rows (%d)",
-                 utf8_char(STRING_ELT(names, j)), element_length, int_nrows);
+                 utf8_char(get_value<r_string_t>(names, j)), element_length, int_nrows);
       }
-      const int* RESTRICT p_is_missing = LOGICAL_RO(is_missing);
+      const r_bool_t* RESTRICT p_is_missing = logical_ptr_ro(is_missing);
       for (int k = 0; k < num_row; ++k){
         p_out[k] += p_is_missing[k];
       }
     } else {
-      const SEXP *p_xj = LIST_PTR_RO(p_x[j]);
+      const SEXP *p_xj = list_ptr_ro(p_x[j]);
       for (int i = 0; i < num_row; ++i){
         p_out[i] += cpp_all_na(p_xj[i], false, true);
       }
@@ -403,27 +397,26 @@ SEXP cpp_df_col_na_counts(SEXP x){
   if (!is_df(x)){
     Rf_error("x must be a data frame");
   }
-  const SEXP *p_x = LIST_PTR_RO(x);
+  const SEXP *p_x = list_ptr_ro(x);
   int num_col = Rf_length(x);
   int32_t NP = 0;
-  int num_row = df_nrow(x);
-  SEXP out = SHIELD(new_vec(INTSXP, num_col)); ++NP;
-  int *p_out = INTEGER(out);
-  std::fill(p_out, p_out + num_col, 0);
+  int num_row = df::nrow(x);
+  SEXP out = SHIELD(vec::new_vector<int>(num_col, 0)); ++NP;
+  int *p_out = integer_ptr(out);
   for (int j = 0; j < num_col; ++j){
     switch ( TYPEOF(p_x[j]) ){
     case VECSXP: {
-      if (Rf_isObject(p_x[j])){
-      SEXP is_missing = SHIELD(cheapr_is_na(p_x[j])); ++NP;
+      if (vec::is_object(p_x[j])){
+      SEXP is_missing = SHIELD(eval_pkg_fun("is_na", "cheapr", env::base_env, p_x[j])); ++NP;
       if (Rf_length(is_missing) != num_row){
         int int_nrows = num_row;
         int element_length = Rf_length(is_missing); ++NP;
-        SEXP names = SHIELD(get_names(x));
+        SEXP names = SHIELD(get_old_names(x));
         YIELD(NP);
         Rf_error("is.na method for list variable %s produces a length (%d) vector which does not equal the number of rows (%d)",
-                 utf8_char(STRING_ELT(names, j)), element_length, int_nrows);
+                 utf8_char(get_value<r_string_t>(names, j)), element_length, int_nrows);
       }
-      int *p_is_missing = LOGICAL(is_missing);
+      int *p_is_missing = integer_ptr(is_missing);
       for (int k = 0; k < num_row; ++k){
         p_out[j] += p_is_missing[k];
       }
@@ -449,31 +442,29 @@ SEXP cpp_col_any_na(SEXP x, bool names){
   if (!is_df(x)){
     Rf_error("x must be a data frame");
   }
-  const SEXP *p_x = LIST_PTR_RO(x);
+  const SEXP *p_x = list_ptr_ro(x);
 
   int32_t NP = 0;
-  int num_row = df_nrow(x);
+  int num_row = df::nrow(x);
   int num_col = Rf_length(x);
 
-  SEXP out = SHIELD(new_vec(LGLSXP, num_col)); ++NP;
-  int *p_out = INTEGER(out);
+  SEXP out = SHIELD(new_vector<r_bool_t>(num_col)); ++NP;
+  int *p_out = integer_ptr(out);
 
   for (int j = 0; j < num_col; ++j){
     switch ( TYPEOF(p_x[j]) ){
     case VECSXP: {
-      if (Rf_isObject(p_x[j])){
-      SEXP is_missing = SHIELD(cheapr_is_na(p_x[j]));
-      cpp11::function r_any = cpp11::package("base")["any"]; ++NP;
+      if (vec::is_object(p_x[j])){
+      SEXP is_missing = SHIELD(eval_pkg_fun("is_na", "cheapr", env::base_env, p_x[j])); ++NP;
       if (Rf_xlength(is_missing) != num_row){
         int int_nrows = num_row;
         int element_length = Rf_xlength(is_missing); ++NP;
-        SEXP names = SHIELD(get_names(x));
+        SEXP names = SHIELD(get_old_names(x));
         YIELD(NP);
         Rf_error("is.na method for list variable %s produces a length (%d) vector which does not equal the number of rows (%d)",
-                 utf8_char(STRING_ELT(names, j)), element_length, int_nrows);
+                 utf8_char(get_value<r_string_t>(names, j)), element_length, int_nrows);
       }
-      SEXP r_any_true = SHIELD(r_any(is_missing)); ++NP;
-      p_out[j] = Rf_asLogical(r_any_true);
+      p_out[j] = static_cast<int>(vec_any(is_missing));
     } else {
       bool any_na = false;
       bool all_na;
@@ -495,9 +486,9 @@ SEXP cpp_col_any_na(SEXP x, bool names){
     }
     }
   }
-  SEXP x_names = SHIELD(get_names(x)); ++NP;
+  SEXP x_names = SHIELD(get_old_names(x)); ++NP;
   if (names){
-    set_names(out, x_names);
+    set_old_names(out, x_names);
   }
   YIELD(NP);
   return out;
@@ -508,32 +499,29 @@ SEXP cpp_col_all_na(SEXP x, bool names){
   if (!is_df(x)){
     Rf_error("x must be a data frame");
   }
-  const SEXP *p_x = LIST_PTR_RO(x);
+  const SEXP *p_x = list_ptr_ro(x);
 
   int32_t NP = 0;
-  int num_row = df_nrow(x);
+  int num_row = df::nrow(x);
   int num_col = Rf_length(x);
 
-  SEXP out = SHIELD(new_vec(LGLSXP, num_col)); ++NP;
-  int *p_out = INTEGER(out);
+  SEXP out = SHIELD(new_vector<r_bool_t>(num_col)); ++NP;
+  int *p_out = integer_ptr(out);
 
   for (int j = 0; j < num_col; ++j){
     switch ( TYPEOF(p_x[j]) ){
     case VECSXP: {
-      if (Rf_isObject(p_x[j])){
-      SEXP is_missing = SHIELD(cheapr_is_na(p_x[j]));
-      cpp11::function r_all = cpp11::package("base")["all"];
-      ++NP;
+      if (vec::is_object(p_x[j])){
+      SEXP is_missing = SHIELD(eval_pkg_fun("is_na", "cheapr", env::base_env, p_x[j])); ++NP;
       if (Rf_xlength(is_missing) != num_row){
         int int_nrows = num_row;
         int element_length = Rf_xlength(is_missing); ++NP;
-        SEXP names = SHIELD(get_names(x));
+        SEXP names = SHIELD(get_old_names(x));
         YIELD(NP);
         Rf_error("is.na method for list variable %s produces a length (%d) vector which does not equal the number of rows (%d)",
-                 utf8_char(STRING_ELT(names, j)), element_length, int_nrows);
+                 utf8_char(get_value<r_string_t>(names, j)), element_length, int_nrows);
       }
-      SEXP r_all_true = SHIELD(r_all(is_missing)); ++NP;
-      p_out[j] = Rf_asLogical(r_all_true);
+      p_out[j] = static_cast<int>(vec_all(is_missing));
     } else {
       bool all_na2 = true;
       bool all_na;
@@ -555,88 +543,23 @@ SEXP cpp_col_all_na(SEXP x, bool names){
     }
     }
   }
-  SEXP x_names = SHIELD(get_names(x)); ++NP;
+  SEXP x_names = SHIELD(get_old_names(x)); ++NP;
   if (names){
-    set_names(out, x_names);
+    set_old_names(out, x_names);
   }
   YIELD(NP);
   return out;
 }
 
-// INCOMPLETE
-//TO-DO - Rewrite this but for all_na()
-// More likely to be faster for the all_na case
-// SEXP cpp_row_any_na(SEXP x, bool names){
-//   if (!is_df(x)){
-//     Rf_error("x must be a data frame");
-//   }
-//   const SEXP *p_x = LIST_PTR_RO(x);
-//
-//   int32_t NP = 0;
-//   int num_row = df_nrow(x);
-//   int num_col = Rf_length(x);
-//
-//   SEXP out = SHIELD(new_vec(LGLSXP, num_row)); ++NP;
-//   int *p_out = INTEGER(out);
-//
-//   bool row_has_na = false;
-//
-//   for (int i = 0; i < num_row; ++i){
-//     row_has_na = false;
-//     for (int j = 0; j < num_col; ++j){
-//       switch ( TYPEOF(p_x[j]) ){
-//       case LGLSXP:
-//       case INTSXP: {
-//         if (is_r_na(INTEGER(p_x[j])[i])){
-//         row_has_na = true;
-//         break;
-//       }
-//         break;
-//       }
-//       case REALSXP: {
-//         if (is_int64(p_x[j])){
-//         int64_t *p_xj = (int64_t *) REAL(p_x[j]);
-//         if (is_r_na(p_xj[i])){
-//           row_has_na = true;
-//           break;
-//         }
-//       } else {
-//         if (is_r_na(REAL(p_x[j])[i])){
-//           row_has_na = true;
-//           break;
-//         }
-//       }
-//         break;
-//       }
-//       case STRSXP: {
-//         if (is_r_na(STRING_PTR_RO(p_x[j])[i])){
-//         row_has_na = true;
-//         break;
-//       }
-//         break;
-//       }
-//       default: {
-//         YIELD(NP);
-//         Rf_error("%s cannot handle an object of type %s", __func__, Rf_type2char(TYPEOF(p_x[j])));
-//       }
-//       }
-//     }
-//     p_out[i] = row_has_na;
-//   }
-//   if (names) cpp_copy_names(x, out, true);
-//   YIELD(NP);
-//   return out;
-// }
-
 R_xlen_t cpp_clean_threshold(double threshold, bool threshold_is_prop, R_xlen_t n){
-  if (threshold != threshold){
+  if (is_r_na(threshold)){
     Rf_error("threshold cannot be NA");
   }
   R_xlen_t out = threshold;
   if (threshold_is_prop){
     if (threshold < 0){
       out = 0;
-    } else if (threshold == R_PosInf){
+    } else if (threshold == r_limits::r_pos_inf){
       out = n + 1;
     } else {
       out = std::floor( (threshold * n) + 0.0000000001);
@@ -645,7 +568,7 @@ R_xlen_t cpp_clean_threshold(double threshold, bool threshold_is_prop, R_xlen_t
     if (threshold < 0){
       out = 0;
     }
-    if (threshold == R_PosInf){
+    if (threshold == r_limits::r_pos_inf){
       out = n + 1;
     }
   }
@@ -663,14 +586,13 @@ SEXP cpp_matrix_row_na_counts(SEXP x){
   R_xlen_t num_row = Rf_nrows(x);
   R_xlen_t num_col = Rf_ncols(x);
   R_xlen_t n = Rf_xlength(x);
-  SEXP out = SHIELD(new_vec(INTSXP, num_row));
-  int *p_out = INTEGER(out);
-  std::fill(p_out, p_out + num_row, 0);
+  SEXP out = SHIELD(vec::new_vector<int>(num_row, 0));
+  int *p_out = integer_ptr(out);
   if (num_row > 0 && num_col > 0){
     switch ( CHEAPR_TYPEOF(x) ){
     case LGLSXP:
     case INTSXP: {
-      int *p_x = INTEGER(x);
+      int *p_x = integer_ptr(x);
       for (R_xlen_t i = 0, rowi = 0; i < n; ++rowi, ++i){
         if (rowi == num_row) rowi = 0;
         p_out[rowi] += is_r_na(p_x[i]);
@@ -678,7 +600,7 @@ SEXP cpp_matrix_row_na_counts(SEXP x){
       break;
     }
     case REALSXP: {
-      double *p_x = REAL(x);
+      double *p_x = real_ptr(x);
       for (R_xlen_t i = 0, rowi = 0; i < n; ++rowi, ++i){
         if (rowi == num_row) rowi = 0;
         p_out[rowi] += is_r_na(p_x[i]);
@@ -686,7 +608,7 @@ SEXP cpp_matrix_row_na_counts(SEXP x){
       break;
     }
     case CHEAPR_INT64SXP: {
-      int64_t *p_x = INTEGER64_PTR(x);
+      int64_t *p_x = integer64_ptr(x);
       for (R_xlen_t i = 0, rowi = 0; i < n; ++rowi, ++i){
         if (rowi == num_row) rowi = 0;
         p_out[rowi] += is_r_na(p_x[i]);
@@ -694,7 +616,7 @@ SEXP cpp_matrix_row_na_counts(SEXP x){
       break;
     }
     case STRSXP: {
-      const SEXP *p_x = STRING_PTR_RO(x);
+      const r_string_t *p_x = string_ptr_ro(x);
       for (R_xlen_t i = 0, rowi = 0; i < n; ++rowi, ++i){
         if (rowi == num_row) rowi = 0;
         p_out[rowi] += is_r_na(p_x[i]);
@@ -705,7 +627,7 @@ SEXP cpp_matrix_row_na_counts(SEXP x){
       break;
     }
     case CPLXSXP: {
-      Rcomplex *p_x = COMPLEX(x);
+      r_complex_t *p_x = complex_ptr(x);
       for (R_xlen_t i = 0, rowi = 0; i < n; ++rowi, ++i){
         if (rowi == num_row) rowi = 0;
         p_out[rowi] += is_r_na(p_x[i]);
@@ -730,14 +652,13 @@ SEXP cpp_matrix_col_na_counts(SEXP x){
   R_xlen_t num_col = Rf_ncols(x);
   R_xlen_t n = Rf_xlength(x);
   bool new_col;
-  SEXP out = SHIELD(new_vec(INTSXP, num_col));
-  int *p_out = INTEGER(out);
-  std::fill(p_out, p_out + num_col, 0);
+  SEXP out = SHIELD(vec::new_vector<int>(num_col, 0));
+  int *p_out = integer_ptr(out);
   if (num_row > 0 && num_col > 0){
     switch ( CHEAPR_TYPEOF(x) ){
     case LGLSXP:
     case INTSXP: {
-      int *p_x = INTEGER(x);
+      int *p_x = integer_ptr(x);
       for (R_xlen_t i = 0, coli = 0, rowi = 0; i < n; ++rowi, ++i){
         new_col = rowi == num_row;
         if (new_col){
@@ -749,7 +670,7 @@ SEXP cpp_matrix_col_na_counts(SEXP x){
       break;
     }
     case REALSXP: {
-      double *p_x = REAL(x);
+      double *p_x = real_ptr(x);
       for (R_xlen_t i = 0, coli = 0, rowi = 0; i < n; ++rowi, ++i){
         new_col = rowi == num_row;
         if (new_col){
@@ -761,7 +682,7 @@ SEXP cpp_matrix_col_na_counts(SEXP x){
       break;
     }
     case CHEAPR_INT64SXP: {
-      int64_t *p_x = INTEGER64_PTR(x);
+      int64_t *p_x = integer64_ptr(x);
       for (R_xlen_t i = 0, coli = 0, rowi = 0; i < n; ++rowi, ++i){
         new_col = rowi == num_row;
         if (new_col){
@@ -773,7 +694,7 @@ SEXP cpp_matrix_col_na_counts(SEXP x){
       break;
     }
     case STRSXP: {
-      const SEXP *p_x = STRING_PTR_RO(x);
+      const r_string_t *p_x = string_ptr_ro(x);
       for (R_xlen_t i = 0, coli = 0, rowi = 0; i < n; ++rowi, ++i){
         new_col = rowi == num_row;
         if (new_col){
@@ -788,7 +709,7 @@ SEXP cpp_matrix_col_na_counts(SEXP x){
       break;
     }
     case CPLXSXP: {
-      Rcomplex *p_x = COMPLEX(x);
+      r_complex_t *p_x = complex_ptr(x);
       for (R_xlen_t i = 0, coli = 0, rowi = 0; i < n; ++rowi, ++i){
         new_col = rowi == num_row;
         if (new_col){
@@ -812,25 +733,26 @@ SEXP cpp_matrix_col_na_counts(SEXP x){
 // Helpers to get matrix row and col names
 
 SEXP matrix_rownames(SEXP x) {
-  SEXP dimnames = SHIELD(Rf_getAttrib(x, R_DimNamesSymbol));
+  SEXP dimnames = SHIELD(get_attr(x, symbol::dim_names_sym));
 
   if (is_null(dimnames) ||
       TYPEOF(dimnames) != VECSXP ||
       Rf_length(dimnames) != 2){
     YIELD(1);
-    return R_NilValue;
+    return r_null;
   }
   YIELD(1);
   return VECTOR_ELT(dimnames, 0);
 }
+
 SEXP matrix_colnames(SEXP x) {
-  SEXP dimnames = SHIELD(Rf_getAttrib(x, R_DimNamesSymbol));
+  SEXP dimnames = SHIELD(get_attr(x, symbol::dim_names_sym));
 
   if (is_null(dimnames) ||
       TYPEOF(dimnames) != VECSXP ||
       Rf_length(dimnames) != 2){
     YIELD(1);
-    return R_NilValue;
+    return r_null;
   }
   YIELD(1);
   return VECTOR_ELT(dimnames, 1);
@@ -851,14 +773,14 @@ SEXP cpp_row_na_counts(SEXP x, bool names){
   if (is_matrix){
     out = SHIELD(cpp_matrix_row_na_counts(x)); ++NP;
     if (names){
-      SEXP row_names = SHIELD(Rf_duplicate(matrix_rownames(x))); ++NP;
-      set_names(out, row_names);
+      SEXP row_names = SHIELD(vec::deep_copy(matrix_rownames(x))); ++NP;
+      set_old_names(out, row_names);
     }
   } else {
     out = SHIELD(cpp_df_row_na_counts(x)); ++NP;
     if (names){
-      SEXP row_names = SHIELD(Rf_duplicate(Rf_getAttrib(x, R_RowNamesSymbol))); ++NP;
-      set_names(out, row_names);
+      SEXP row_names = SHIELD(vec::deep_copy(get_attr(x, symbol::row_names_sym))); ++NP;
+      set_old_names(out, row_names);
     }
   }
   YIELD(NP);
@@ -880,14 +802,14 @@ SEXP cpp_col_na_counts(SEXP x, bool names){
   if (is_matrix){
     out = SHIELD(cpp_matrix_col_na_counts(x)); ++NP;
     if (names){
-      SEXP col_names = SHIELD(Rf_duplicate(matrix_colnames(x))); ++NP;
-      set_names(out, col_names);
+      SEXP col_names = SHIELD(vec::deep_copy(matrix_colnames(x))); ++NP;
+      set_old_names(out, col_names);
     }
   } else {
     out = SHIELD(cpp_df_col_na_counts(x)); ++NP;
-    SEXP x_names = SHIELD(get_names(x)); ++NP;
+    SEXP x_names = SHIELD(get_old_names(x)); ++NP;
     if (names){
-      set_names(out, x_names);
+      set_old_names(out, x_names);
     }
   }
   YIELD(NP);
