@@ -37,7 +37,7 @@ test_that("data frames", {
   )
   expect_identical(
     as_df(matrix(1:10, ncol = 2)),
-    new_df(col_1 = 1:5, col_2 = 6:10)
+    new_df(value = 1:10)
   )
 
   expect_identical(
